"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var adapter_sqlite_nativescript_1 = require("@nano-sql/adapter-sqlite-nativescript");
var index_1 = require("@nano-sql/core/lib/index");
var planned_task_1 = require("../tasks/planner/planned-task");
var time_1 = require("../utils/time");
var DB_NAME = "tasks-dispatcher";
var PLANNED_TASKS_TABLE = "plannedTasks";
var PlannedTaskDBStore = (function () {
    function PlannedTaskDBStore() {
        this.dbInitialized = false;
    }
    PlannedTaskDBStore.prototype.insert = function (plannedTask) {
        return __awaiter(this, void 0, void 0, function () {
            var name, startAt, interval, recurrent, params, cancelEvent, runnableTask, possibleTask, instance;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        name = plannedTask.name, startAt = plannedTask.startAt, interval = plannedTask.interval, recurrent = plannedTask.recurrent, params = plannedTask.params, cancelEvent = plannedTask.cancelEvent;
                        runnableTask = {
                            name: name,
                            startAt: startAt,
                            interval: interval,
                            recurrent: recurrent,
                            params: params,
                            cancelEvent: cancelEvent,
                        };
                        return [4, this.get(runnableTask)];
                    case 1:
                        possibleTask = _a.sent();
                        if (possibleTask) {
                            throw new PlannedTaskAlreadyExistsError(plannedTask);
                        }
                        return [4, this.db()];
                    case 2:
                        instance = _a.sent();
                        return [4, instance.query("upsert", __assign({}, plannedTask)).exec()];
                    case 3:
                        _a.sent();
                        return [2];
                }
            });
        });
    };
    PlannedTaskDBStore.prototype.delete = function (taskId) {
        return __awaiter(this, void 0, void 0, function () {
            var instance;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, this.db()];
                    case 1:
                        instance = _a.sent();
                        return [4, instance.query("delete").where(["id", "=", taskId]).exec()];
                    case 2:
                        _a.sent();
                        return [2];
                }
            });
        });
    };
    PlannedTaskDBStore.prototype.get = function (task) {
        return __awaiter(this, void 0, void 0, function () {
            var e_1, _a, whereStatement, runnableTask, instance, rows, params, rows_1, rows_1_1, row;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        whereStatement = ["id", "=", task];
                        if (typeof task !== "string") {
                            runnableTask = task;
                            whereStatement = [
                                ["name", "=", runnableTask.name],
                                "AND",
                                ["startAt", "=", runnableTask.startAt],
                                "AND",
                                ["interval", "=", runnableTask.interval],
                                "AND",
                                ["recurrent", "=", runnableTask.recurrent],
                            ];
                        }
                        return [4, this.db()];
                    case 1:
                        instance = _b.sent();
                        index_1.nSQL(PLANNED_TASKS_TABLE).useDatabase(DB_NAME);
                        return [4, instance.query("select").where(whereStatement).exec()];
                    case 2:
                        rows = _b.sent();
                        if (rows.length === 0) {
                            return [2, null];
                        }
                        if (typeof task === "string") {
                            return [2, this.plannedTaskFromRow(rows[0])];
                        }
                        params = JSON.stringify(task.params);
                        try {
                            for (rows_1 = __values(rows), rows_1_1 = rows_1.next(); !rows_1_1.done; rows_1_1 = rows_1.next()) {
                                row = rows_1_1.value;
                                if (params === JSON.stringify(row.params)) {
                                    return [2, this.plannedTaskFromRow(row)];
                                }
                            }
                        }
                        catch (e_1_1) { e_1 = { error: e_1_1 }; }
                        finally {
                            try {
                                if (rows_1_1 && !rows_1_1.done && (_a = rows_1.return)) _a.call(rows_1);
                            }
                            finally { if (e_1) throw e_1.error; }
                        }
                        return [2, null];
                }
            });
        });
    };
    PlannedTaskDBStore.prototype.getAllSortedByNextRun = function (planningType) {
        return __awaiter(this, void 0, void 0, function () {
            var instance, query, rows, plannedTasks, currentMillis;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, this.db()];
                    case 1:
                        instance = _a.sent();
                        query = instance.query("select");
                        if (planningType) {
                            query = query.where(["planningType", "=", planningType]);
                        }
                        return [4, query.exec()];
                    case 2:
                        rows = _a.sent();
                        plannedTasks = rows.map(function (row) { return _this.plannedTaskFromRow(row); });
                        currentMillis = time_1.now();
                        return [2, plannedTasks.sort(function (t1, t2) { return t1.nextRun(currentMillis) - t2.nextRun(currentMillis); })];
                }
            });
        });
    };
    PlannedTaskDBStore.prototype.getAllCancelEvents = function () {
        return __awaiter(this, void 0, void 0, function () {
            var instance, rows;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, this.db()];
                    case 1:
                        instance = _a.sent();
                        return [4, instance
                                .query("select")
                                .distinct(["cancelEvent"])
                                .exec()];
                    case 2:
                        rows = _a.sent();
                        return [2, rows.map(function (row) { return row.cancelEvent; })];
                }
            });
        });
    };
    PlannedTaskDBStore.prototype.getAllFilteredByCancelEvent = function (cancelEvent) {
        return __awaiter(this, void 0, void 0, function () {
            var instance, rows;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, this.db()];
                    case 1:
                        instance = _a.sent();
                        return [4, instance
                                .query("select")
                                .where(["cancelEvent", "=", cancelEvent])
                                .exec()];
                    case 2:
                        rows = _a.sent();
                        return [2, rows.map(function (row) { return _this.plannedTaskFromRow(row); })];
                }
            });
        });
    };
    PlannedTaskDBStore.prototype.increaseErrorCount = function (taskId) {
        return __awaiter(this, void 0, void 0, function () {
            var plannedTask, instance;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, this.get(taskId)];
                    case 1:
                        plannedTask = _a.sent();
                        if (!plannedTask) return [3, 4];
                        return [4, this.db(PLANNED_TASKS_TABLE + ".errorCount")];
                    case 2:
                        instance = _a.sent();
                        return [4, instance
                                .query("upsert", plannedTask.errorCount + 1)
                                .where(["id", "=", taskId])
                                .exec()];
                    case 3:
                        _a.sent();
                        return [3, 5];
                    case 4: throw new Error("Task not found: " + taskId);
                    case 5: return [2];
                }
            });
        });
    };
    PlannedTaskDBStore.prototype.increaseTimeoutCount = function (taskId) {
        return __awaiter(this, void 0, void 0, function () {
            var plannedTask, instance;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, this.get(taskId)];
                    case 1:
                        plannedTask = _a.sent();
                        if (!plannedTask) return [3, 4];
                        return [4, this.db(PLANNED_TASKS_TABLE + ".timeoutCount")];
                    case 2:
                        instance = _a.sent();
                        return [4, instance
                                .query("upsert", plannedTask.timeoutCount + 1)
                                .where(["id", "=", taskId])
                                .exec()];
                    case 3:
                        _a.sent();
                        return [3, 5];
                    case 4: throw new Error("Task not found: " + taskId);
                    case 5: return [2];
                }
            });
        });
    };
    PlannedTaskDBStore.prototype.updateLastRun = function (taskId, timestamp) {
        return __awaiter(this, void 0, void 0, function () {
            var plannedTask, instance;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, this.get(taskId)];
                    case 1:
                        plannedTask = _a.sent();
                        if (!plannedTask) return [3, 4];
                        return [4, this.db(PLANNED_TASKS_TABLE + ".lastRun")];
                    case 2:
                        instance = _a.sent();
                        return [4, instance
                                .query("upsert", timestamp)
                                .where(["id", "=", taskId])
                                .exec()];
                    case 3:
                        _a.sent();
                        return [3, 5];
                    case 4: throw new Error("Task not found: " + taskId);
                    case 5: return [2];
                }
            });
        });
    };
    PlannedTaskDBStore.prototype.deleteAll = function () {
        return __awaiter(this, void 0, void 0, function () {
            var instance;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, this.db()];
                    case 1:
                        instance = _a.sent();
                        return [4, instance.query("delete").exec()];
                    case 2:
                        _a.sent();
                        return [2];
                }
            });
        });
    };
    PlannedTaskDBStore.prototype.db = function (tableName) {
        if (tableName === void 0) { tableName = PLANNED_TASKS_TABLE; }
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, this.createDB()];
                    case 1:
                        _a.sent();
                        if (index_1.nSQL().selectedDB !== DB_NAME) {
                            index_1.nSQL().useDatabase(DB_NAME);
                        }
                        return [2, index_1.nSQL(tableName)];
                }
            });
        });
    };
    PlannedTaskDBStore.prototype.createDB = function () {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (this.dbInitialized) {
                            return [2];
                        }
                        if (!this.createDBProcedure) {
                            this.createDBProcedure = index_1.nSQL().createDatabase({
                                id: DB_NAME,
                                mode: new adapter_sqlite_nativescript_1.NativeSQLite(),
                                tables: [
                                    {
                                        name: PLANNED_TASKS_TABLE,
                                        model: {
                                            "id:uuid": { pk: true },
                                            "planningType:string": {},
                                            "schedulerType:string": {},
                                            "name:string": {},
                                            "startAt:int": {},
                                            "params:obj": {},
                                            "interval:int": {},
                                            "recurrent:boolean": {},
                                            "createdAt:int": {},
                                            "errorCount:int": {},
                                            "timeoutCount:int": {},
                                            "lastRun:int": {},
                                            "cancelEvent:string": {},
                                        },
                                    },
                                ],
                            });
                        }
                        return [4, this.createDBProcedure];
                    case 1:
                        _a.sent();
                        this.dbInitialized = true;
                        return [2];
                }
            });
        });
    };
    PlannedTaskDBStore.prototype.plannedTaskFromRow = function (obj) {
        return new planned_task_1.PlannedTask(obj.planningType, obj.schedulerType, {
            name: obj.name,
            startAt: obj.startAt,
            interval: obj.interval,
            recurrent: obj.recurrent,
            params: obj.params,
            cancelEvent: obj.cancelEvent,
        }, obj.id, obj.createdAt, obj.lastRun, obj.errorCount, obj.timeoutCount);
    };
    return PlannedTaskDBStore;
}());
exports.plannedTasksDB = new PlannedTaskDBStore();
var PlannedTaskAlreadyExistsError = (function (_super) {
    __extends(PlannedTaskAlreadyExistsError, _super);
    function PlannedTaskAlreadyExistsError(_a) {
        var name = _a.name, startAt = _a.startAt, interval = _a.interval, recurrent = _a.recurrent;
        return _super.call(this, "Already stored: {name=" + name + ", startAt=" + startAt + ", interval=" + interval + ", recurrent=" + recurrent + "}") || this;
    }
    return PlannedTaskAlreadyExistsError;
}(Error));
exports.PlannedTaskAlreadyExistsError = PlannedTaskAlreadyExistsError;
//# sourceMappingURL=planned-tasks-store.js.map